@@ -14,11 +14,11 @@ function(fit,scope=eval(fit$call$data), alpha = 0.05,direction = "forward",crite
     if(direction != "forward" && direction != "backward" && direction != "both"){
         stop("\ndirection should be one of the following: forward, backward, and both\n")
     }
-    if(criterion != "p-value" && criterion != "AIC" && criterion != "BIC" && criterion != "r-adj"){
-        stop("\ncriterion should be one of the following: p-value, AIC, BIC, and r-adj\n")
+    if(criterion != "p-value" && criterion != "AIC" && criterion != "BIC" && criterion != "r-adj" && criterion != "PRESS"){
+        stop("\ncriterion should be one of the following: p-value, AIC, BIC, r-adj or PRESS\n")
     }
     if(correction != "FDR" && correction != "Bonferroni" && correction !="Bonf" && correction != "None"){
-        stop("\ncorrection should be one of the following: FDR, Bonferroni, Bonf, None \n")
+        stop("\ncorrection should be one of the following: FDR, Bonferroni, Bonf, or None \n")
     }
     if(alpha>1 || alpha <0){
         stop("\nSignificance level alpha should be smaller than 1 and larger than 0\n")
@@ -51,9 +51,61 @@ function(fit,scope=eval(fit$call$data), alpha = 0.05,direction = "forward",crite
         return(result) #when satisfy corrected criterion return result=TRUE, otherwise result=FALSE
     }
     
+    #######################PRESS residuals########################
+	PRESS <- function(linear.model) {
+	  #' calculate the predictive residuals
+	  pr <- residuals(linear.model)/(1-lm.influence(linear.model)$hat)
+	  #' calculate the PRESS
+	  PRESS <- sum(pr^2)
+	  return(PRESS)
+	}
+
+    ####################### VIF ########################
+	vif <- function(mod, ...) 
+	{
+	    if (any(is.na(coef(mod)))) 
+	        stop ("there are aliased coefficients in the model")
+	    v <- vcov(mod)
+	    assign <- attr(model.matrix(mod), "assign")
+	    if (names(coefficients(mod)[1]) == "(Intercept)") {
+	        v <- v[-1, -1]
+	        assign <- assign[-1]
+	    }
+	    else warning("No intercept: vifs may not be sensible.")
+	    terms <- labels(terms(mod))
+	    n.terms <- length(terms)
+	    if (n.terms < 2) stop("model contains fewer than 2 terms")
+	    R <- cov2cor(v)
+	    detR <- det(R)
+	    result <- matrix(0, n.terms, 3)
+	    rownames(result) <- terms
+	    colnames(result) <- c("GVIF", "Df", "GVIF^(1/(2*Df))")
+	    for (term in 1:n.terms) {
+	        subs <- which(assign == term)
+	        result[term, 1] <- det(as.matrix(R[subs, subs])) *
+	            det(as.matrix(R[-subs, -subs])) / detR
+	        result[term, 2] <- length(subs)
+	    }
+	    if (all(result[, 2] == 1)) result <- result[, 1]
+	    else result[, 3] <- result[, 1]^(1/(2 * result[, 2]))
+	    result
+	}
+
+
+	if(criterion == "p-value")
+		crit_column_to_sort = 6 ## this is used to sort and present the out_tab (ouput table) sorted by this criterion
+	else if(criterion == "AIC")
+		crit_column_to_sort = 2
+	else if(criterion == "BIC") 
+		crit_column_to_sort = 3
+	else if(criterion == "r-adj")
+		crit_column_to_sort = 4
+    else if(criterion == "PRESS")
+	    crit_column_to_sort = 5
+
     #########################forward1 function#############################
     
-    forward1 <- function(fit,override=FALSE){
+    forward1 <- function(fit){
         data <- eval(fit$call$data) #save data
         response <- fit$terms[[2]] #reponse variable
         reg_var <- variable.names(fit)[-1] #save all the dependent vars of 'fit' model
@@ -99,22 +151,28 @@ function(fit,scope=eval(fit$call$data), alpha = 0.05,direction = "forward",crite
             out_tab <- NA
             
         }else{
-            out_tab <- matrix(nrow=length(var)+1, ncol=8) #outcome table
-            colnames(out_tab) <- c("RSS", "AIC", "BIC", "adj.rsq","max_pvalue", "alpha_cut_off", "Bonferroni", "FDR")
+            out_tab <- matrix(nrow=length(var)+1, ncol=10) #outcome table
+            colnames(out_tab) <- c("RSS", "AIC", "BIC", "adj.rsq","PRESS", "max_pvalue", "max VIF", "alpha_cut_off", "Bonferroni", "FDR")
             rownames(out_tab) <- c("<none>", paste("+", var))
             out_tab[1,1] <- deviance(fit) #RSS of the current model
             out_tab[1,2] <- AIC(fit) #AIC of the current model
             out_tab[1,3] <- BIC(fit) #BIC of the current model
             out_tab[1,4] <- summary(fit)$"adj.r.squared" #adjusted r-square of the current model
+            out_tab[1,5] <- PRESS(fit)
+            
+            if (length(fit$coefficients) > 2) ## one of them is beta0, so need 3 or more coef
+                out_tab[1,7] <- max(vif(fit)) 
+            else
+                out_tab[1,7] <- NA
             
             fit_pval <- drop1(fit, test="F")$"Pr(>F)"[-1] #save p-values of the current model
             if(length(fit_pval) == 0){ #for the null model
-                out_tab[1,5] <- out_tab[1,6] <- out_tab[1,7] <-out_tab[1,8] <- NA
+                out_tab[1,6] <- out_tab[1,8] <-out_tab[1,9] <-out_tab[1,10] <- NA
             }else{
-                out_tab[1,5] <- max(fit_pval) #the max p-value of the current model
-                out_tab[1,6] <- none(fit_pval) #pvalue cut-off (no correction)
-                out_tab[1,7] <- bonferroni(fit_pval) #Bonferroni cut-off
-                out_tab[1,8] <- fdr(fit_pval) #FDR cut-off
+                out_tab[1,6] <- max(fit_pval) #the max p-value of the current model
+                out_tab[1,8] <- none(fit_pval) #pvalue cut-off (no correction)
+                out_tab[1,9] <- bonferroni(fit_pval) #Bonferroni cut-off
+                out_tab[1,10] <- fdr(fit_pval) #FDR cut-off
             }
             crit_value <- matrix() #criterion values
             if(criterion == "p-value"){
@@ -125,6 +183,8 @@ function(fit,scope=eval(fit$call$data), alpha = 0.05,direction = "forward",crite
                 crit_value[1] <- out_tab[1,3] #save BIC as criterion value #########
             } else if(criterion == "r-adj"){
                 crit_value[1] <- -out_tab[1,4] #save -adjusted r-square
+            } else if(criterion == "PRESS"){
+                crit_value[1] <- out_tab[1,5] #save -PRESS
             }
             
             for (n in 1:length(var)){
@@ -134,79 +194,85 @@ function(fit,scope=eval(fit$call$data), alpha = 0.05,direction = "forward",crite
                 out_tab[n+1,2] <- AIC(fit2) #save value of AIC value for each model ##########
                 out_tab[n+1,3] <- BIC(fit2) #save value of BIC value for each model ##########
                 out_tab[n+1,4] <- summary(fit2)$"adj.r.squared" #save adjusted r-square
+                out_tab[n+1,5] <- PRESS(fit2)
+
+	            if (length(fit2$coefficients) > 2) ## one of them is beta0, so need 3 or more coef
+    	            out_tab[n+1,7] <- max(vif(fit2))
+    	        else
+    	            out_tab[n+1,7] <- NA
                 
                 fit2_pval <- drop1(fit2, test="F")$"Pr(>F)"[-1] #save p-values of each model
-                out_tab[n+1,5] <- max(fit2_pval) #the max p-value of each model
-                out_tab[n+1,6] <- none(fit2_pval) #pvalue cut-off (no correction)
-                out_tab[n+1,7] <- bonferroni(fit2_pval) #Bonferroni cut-off
-                out_tab[n+1,8] <- fdr(fit2_pval) #FDR cut-off
+                out_tab[n+1,6] <- max(fit2_pval) #the max p-value of each model
+                out_tab[n+1,8] <- none(fit2_pval) #pvalue cut-off (no correction)
+                out_tab[n+1,9] <- bonferroni(fit2_pval) #Bonferroni cut-off
+                out_tab[n+1,10] <- fdr(fit2_pval) #FDR cut-off
                 
                 if (criterion == "p-value"){
-                    crit_value[n+1] <- out_tab[n+1,5] #save the biggest p-value of each model
+                    crit_value[n+1] <- out_tab[n+1,6] #save the biggest p-value of each model
                 } else if (criterion == "AIC"){
                     crit_value[n+1] <- out_tab[n+1,2] #save AIC value of each model ##########
                 } else if (criterion == "BIC"){
                     crit_value[n+1] <- out_tab[n+1,3] #save BIC value of each model ##########
                 } else if (criterion == "r-adj"){
                     crit_value[n+1] <- -out_tab[n+1,4] #save -adjusted r-square
+                } else if (criterion == "PRESS"){
+                    crit_value[n+1] <- out_tab[n+1,5] #save -adjusted r-square
                 }
             }
             
+            out_tab = out_tab[order(as.numeric(out_tab[,crit_column_to_sort])),]
             out_tab <- round(as.table(out_tab, dimnames),5)
-            out_tab[,6:8] <- as.character(as.logical(out_tab[,6:8]))
+            out_tab[,8:10] <- as.character(as.logical(out_tab[,8:10]))
             #if(trace==TRUE){cat("\n"); print(out_tab)}
             
             crit_value[which(is.na(crit_value=="NA"))]  <- 99 #remove the model of which p-value is NA
             ##################################
             
             var_index <- which(crit_value==min(crit_value[-1]))-1
-            if(length(var_index)>1) var_index <- var_index[1] #when p-value is the same(models for tie), select the first one
+            if(length(var_index)>1) #when p-value is the same(models for tie), select the first one
+            if (criterion == "p-value")
+            {
+            	F_value = 0
+            	for (n in 1:length(var_index))
+            		F_value[n] = summary(update(fit, paste(".~. +", var[var_index[n]])))$fstatistic[1]
+            	var_index = which.max(F_value)
+            } else var_index <- var_index[1]          
             
             fit2 <- update(fit, paste(".~. +", var[var_index]))
             pvalues <- drop1(fit2,test="F")$"Pr(>F)"[-1] #p-values
             
-            if(override == FALSE){
-                if(correction=="FDR"){
-                    result <- fdr(pvalues)
-                }else if(correction=="Bonferroni" | correction=="Bonf"){
-                    result <- bonferroni(pvalues) #Bonferroni cut-off
-                }else if(correction=="None"){
-                    result <- none(pvalues)
-                }
-                if(criterion == "p-value"){
-                    if(direction == "forward"){
-                        if(result == TRUE){
-                            fit <- fit2
-                            switch <- TRUE
-                        }else if(result == FALSE){
-                            #fit <- fit
-                            switch <- FALSE
-                        }
-                    }else if(direction == "both"){
+            if(correction=="FDR"){
+                result <- fdr(pvalues)
+            }else if(correction=="Bonferroni" | correction=="Bonf"){
+                result <- bonferroni(pvalues) #Bonferroni cut-off
+            }else if(correction=="None"){
+                result <- none(pvalues)
+            }
+            if(criterion == "p-value"){
+                if(direction == "forward"){
+                    if(result == TRUE){
                         fit <- fit2
-                        switch <- result #save correction1 result
+                        switch <- TRUE
+                    }else if(result == FALSE){
+                        #fit <- fit
+                        switch <- FALSE
                     }
-                }else if(criterion == "AIC" | criterion =="BIC" | criterion =="r-adj"){
-                    if(direction == "forward"){
-                        if(result == TRUE & min(crit_value[-1]) < crit_value[1]){
-                            fit <- fit2
-                            switch <- TRUE
-                        }else{
-                            #fit <- fit
-                            switch <- FALSE
-                        }
-                    }else if(direction == "both"){
-                        fit <- fit2
-                        switch <- result #save correction1 result
-                    }
-                }
-            }else if(override == TRUE){ #do not check correction)
-                if(criterion == "p-value"){
+                }else if(direction == "both"){
                     fit <- fit2
-                }else if(criterion == "AIC" | criterion =="BIC" | criterion =="r-adj"){
-                    if(min(crit_value[-1]) > crit_value[1]){
-                        fit <- fit
-                    }else fit <- fit2
+                    switch <- result #save correction1 result
+                }
+            }else if(criterion == "AIC" | criterion =="BIC" | criterion =="r-adj" | criterion =="PRESS"){
+                if(direction == "forward"){
+                    if(result == TRUE & min(crit_value[-1]) < crit_value[1]){
+                        fit <- fit2
+                        switch <- TRUE
+                    }else{
+                        #fit <- fit
+                        switch <- FALSE
+                    }
+                }else if(direction == "both"){
+                    fit <- fit2
+                    switch <- result #save correction1 result
                 }
             }
         }
@@ -217,7 +283,7 @@ function(fit,scope=eval(fit$call$data), alpha = 0.05,direction = "forward",crite
     
     ######################################################################
     #########################backward1 function############################
-    backward1 <- function(fit, override=FALSE){
+    backward1 <- function(fit){
         data <- eval(fit$call$data) #save data
         response <- fit$terms[[2]] #reponse variable
         reg_var <- variable.names(fit)[-1] #save all the dependent vars of 'fit' model
@@ -257,19 +323,25 @@ function(fit,scope=eval(fit$call$data), alpha = 0.05,direction = "forward",crite
             out_tab <- NA
         }else{
             if(length(reg_var) !=0){
-                out_tab <- matrix(nrow=length(var)+1, ncol=8) #outcome table
-                colnames(out_tab) <- c("RSS", "AIC", "BIC", "adj.rsq","max_pvalue", "alpha_cut-off", "Bonferroni", "FDR")
+                out_tab <- matrix(nrow=length(var)+1, ncol=10) #outcome table
+                colnames(out_tab) <- c("RSS", "AIC", "BIC", "adj.rsq","PRESS", "max_pvalue", "max VIF", "alpha_cut-off", "Bonferroni", "FDR")
                 rownames(out_tab) <- c("<none>", paste("-", var))
                 out_tab[1,1] <- deviance(fit) #RSS of the current model
                 out_tab[1,2] <- AIC(fit) #AIC of the current model
                 out_tab[1,3] <- BIC(fit) #BIC of the current model
                 out_tab[1,4] <- summary(fit)$"adj.r.squared" #adjusted r-square of the current model
+                out_tab[1,5] <- PRESS(fit) #PRESS of the current model
                 
+	            if (length(fit$coefficients) > 2) ## one of them is beta0, so need 3 or more coef
+	                out_tab[1,7] <- max(vif(fit)) 
+	            else
+	                out_tab[1,7] <- NA
+
                 fit_pval <- drop1(fit, test="F")$"Pr(>F)"[-1] #save p-values of the current model
-                out_tab[1,5] <- max(fit_pval) #the max p-value of the current model
-                out_tab[1,6] <- none(fit_pval) #pvalue cut-off (no correction)
-                out_tab[1,7] <- bonferroni(fit_pval) #Bonferroni cut-off
-                out_tab[1,8] <- fdr(fit_pval) #FDR cut-off
+                out_tab[1,6] <- max(fit_pval) #the max p-value of the current model
+                out_tab[1,8] <- none(fit_pval) #pvalue cut-off (no correction)
+                out_tab[1,9] <- bonferroni(fit_pval) #Bonferroni cut-off
+                out_tab[1,10] <- fdr(fit_pval) #FDR cut-off
                 
                 crit_value <- matrix() #criterion values
                 if(criterion == "p-value"){
@@ -280,6 +352,8 @@ function(fit,scope=eval(fit$call$data), alpha = 0.05,direction = "forward",crite
                     crit_value[1] <- out_tab[1,3] #save BIC as criterion value #########
                 } else if(criterion == "r-adj"){
                     crit_value[1] <- -out_tab[1,4] #save -adjusted r-square
+                } else if(criterion == "PRESS"){
+                    crit_value[1] <- out_tab[1,5] #save -adjusted r-square
                 }
                 
                 for (n in 1:length(var)){
@@ -289,33 +363,42 @@ function(fit,scope=eval(fit$call$data), alpha = 0.05,direction = "forward",crite
                     out_tab[n+1,2] <- AIC(fit2) #save value of AIC value for each model ##########
                     out_tab[n+1,3] <- BIC(fit2) #save value of BIC value for each model ##########
                     out_tab[n+1,4] <- summary(fit2)$"adj.r.squared" #save adjusted r-square
+                    out_tab[n+1,5] <- PRESS(fit2) #save adjusted r-square
+
+		            if (length(fit2$coefficients) > 2) ## one of them is beta0, so need 3 or more coef
+                    	out_tab[n+1,7] <- max(vif(fit2)) 
+                    else
+                    	out_tab[n+1,7] <- NA
                     
                     fit2_pval <- drop1(fit2, test="F")$"Pr(>F)"[-1] #save p-values of each model
                     #if(length(fit2_pval)==0)  stop("\nwrong scope or outcome is a null model\n")
                     if(length(fit2_pval)==0){ #if fit2 is a null model
-                        out_tab[n+1,5] <- NA
                         out_tab[n+1,6] <- NA
-                        out_tab[n+1,7] <- NA
                         out_tab[n+1,8] <- NA
+                        out_tab[n+1,9] <- NA
+                        out_tab[n+1,10] <- NA
                     }else{
-                        out_tab[n+1,5] <- max(fit2_pval) #the max p-value of each model
-                        out_tab[n+1,6] <- none(fit2_pval) #pvalue cut-off (no correction)
-                        out_tab[n+1,7] <- bonferroni(fit2_pval) #Bonferroni cut-off
-                        out_tab[n+1,8] <- fdr(fit2_pval) #FDR cut-off
+                        out_tab[n+1,6] <- max(fit2_pval) #the max p-value of each model
+                        out_tab[n+1,8] <- none(fit2_pval) #pvalue cut-off (no correction)
+                        out_tab[n+1,9] <- bonferroni(fit2_pval) #Bonferroni cut-off
+                        out_tab[n+1,10] <- fdr(fit2_pval) #FDR cut-off
                     }
                     if (criterion == "p-value"){
-                        crit_value[n+1] <- out_tab[n+1,5] #save the biggest p-value of each model
+                        crit_value[n+1] <- out_tab[n+1,6] #save the biggest p-value of each model
                     } else if (criterion == "AIC"){
                         crit_value[n+1] <- out_tab[n+1,2] #save AIC value of each model ##########
                     } else if (criterion == "BIC"){
                         crit_value[n+1] <- out_tab[n+1,3] #save BIC value of each model ##########
                     } else if (criterion == "r-adj"){
                         crit_value[n+1] <- -out_tab[n+1,4] #save -adjusted r-square
+                    } else if (criterion == "PRESS"){
+                        crit_value[n+1] <- out_tab[n+1,5] #save -adjusted r-square
                     }
                 }
                 
+	            out_tab = out_tab[order(as.numeric(out_tab[,crit_column_to_sort])),]
                 out_tab <- round(as.table(out_tab, dimnames),5)
-                out_tab[,6:8] <- as.character(as.logical(out_tab[,6:8]))
+                out_tab[,8:10] <- as.character(as.logical(out_tab[,8:10]))
                 #if(trace==TRUE){cat("\n"); print(out_tab)}
                 
                 crit_value[which(is.na(crit_value=="NA"))]  <- 99 #remove the model of which p-value is NA
@@ -328,94 +411,41 @@ function(fit,scope=eval(fit$call$data), alpha = 0.05,direction = "forward",crite
                 }else fit2 <- update(fit, paste(".~. -", var[var_index]))
                 pvalues <- drop1(fit2,test="F")$"Pr(>F)"[-1] #p-values
                 
-                if(override == FALSE){
-                    if(length(pvalues)==0){ #if fit2 is a null model, donot check the correction
-                        result <- TRUE
-                    }else{ #if fit2 is not a null model, check the correction
-                        if(correction=="FDR"){
-                            result <- fdr(pvalues)
-                        }else if(correction=="Bonferroni" | correction=="Bonf"){
-                            result <- bonferroni(pvalues) #Bonferroni cut-off
-                        }else if(correction=="None"){
-                            result <- none(pvalues)
-                        }
+                if(length(pvalues)==0){ #if fit2 is a null model, donot check the correction
+                    result <- TRUE
+                }else{ #if fit2 is not a null model, check the correction
+                    if(correction=="FDR"){
+                        result <- fdr(pvalues)
+                    }else if(correction=="Bonferroni" | correction=="Bonf"){
+                        result <- bonferroni(pvalues) #Bonferroni cut-off
+                    }else if(correction=="None"){
+                        result <- none(pvalues)
                     }
-                    if(criterion == "p-value"){
-                        if (direction == "backward"){
-                            if (length(reg_var)>1){ # if 'fit' model has more than one predictor
-                                if(result == FALSE){
-                                    fit <- fit2
-                                    switch <- TRUE #when a correction result is FALSE, run the while loop
-                                }else if(result == TRUE){
-                                    
-                                    if(correction=="FDR"){ #check the current model is significant with correction
-                                        if(out_tab[1,8] == TRUE){ #if the current model is already significant even at the beginning
-                                            switch <- FALSE
-                                        }else{fit<-fit2; switch <- TRUE}  # if the current model is not significant, but fit2 is significant
-                                    }else if(correction=="Bonferroni" | correction=="Bonf"){
-                                        if(out_tab[1,7] == TRUE){
-                                            switch <- FALSE
-                                        }else{fit<-fit2; switch <- TRUE} # if the current model is not significant, but fit2 is significant
-                                    }else if(correction=="None"){ #if the current model is already significant
-                                        if(out_tab[1,6] == FALSE){
-                                            switch <- FALSE
-                                        }else{fit<-fit2; switch <- TRUE}
-                                    }
-                                    
-                                }
-                            }else if (length(reg_var)==1){ #fit model has only one variable
-                                if (result == TRUE){ #when the model has only one predictor
-                                    if(correction=="FDR"){ #check the only predictor is significant with correction
-                                        if(out_tab[1,8] == FALSE){
-                                            fit <- fit2; switch <- FALSE #########################
-                                        }else switch <- FALSE # if only predictor is significant, stop the while loop
-                                    }else if(correction=="Bonferroni" | correction=="Bonf"){
-                                        if(out_tab[1,7] == FALSE){
-                                            fit <- fit2; switch <- FALSE
-                                        }else switch <- FALSE # if only predictor is significant, stop the while loop
-                                    }else if(correction=="None"){
-                                        if(out_tab[1,6] == FALSE){
-                                            fit <- fit2; switch <- FALSE
-                                        }else switch <- FALSE # if only predictor is significant, stop the while loop
-                                    }
-                                    #switch <- FALSE # if only predictor is significant, stop the while loop
-                                }else if(length(var == 1)) { #when result is false and scope has one variable, make a null model
-                                    fit2 <- update(fit, paste(".~. -", var[n]))
-                                    switch <- FALSE
-                                    #stop("\noutcome is a null model\n")
-                                }else{ #should keep a variable not in the scope even if it is not significant
-                                    fit2 <- fit
-                                    switch <- FALSE
-                                }
-                            }else switch <- FALSE #when no predictor is left, stop the while loop
-                        }else if(direction == "both"){
+                }
+                if(criterion == "p-value"){
+                    if (direction == "backward")
+                    {
                             fit <- fit2
-                            switch <- result #send correction1 result as switch
-                        }
-                    }else if(criterion == "AIC" | criterion =="BIC" | criterion =="r-adj"){
-                        if(direction=="backward"){
-                            if(result == FALSE | min(crit_value[-1]) < crit_value[1]){
-                                fit <- fit2
-                                switch <- TRUE #run the while loop
-                            }else if(length(variable.names(fit2)[-1])==0){ #if fit2 is a null model and fit is better than fit2
-                                #fit <- fit
+                        	if (result == FALSE)  
+                                switch <- TRUE
+                            else
                                 switch <- FALSE
-                            }else{
-                                fit <- fit2
-                                switch <- FALSE #escape the while loop when there is no variable decreasing AIC when dropped
-                            }
-                        }else if(direction == "both"){
-                            fit <- fit2
-                            switch <- result #send correction1 result as switch
-                        }
-                    }
-                }else if(override == TRUE){ #do not check correction)
-                    if(criterion == "p-value"){
+                    }else if(direction == "both"){
                         fit <- fit2
-                    }else if(criterion == "AIC" | criterion =="BIC" | criterion =="r-adj"){
-                        if(min(crit_value[-1]) > crit_value[1]){
-                            fit <- fit
-                        }else fit <- fit2
+                        switch <- result #send correction1 result as switch
+                    }
+                }else if(criterion == "AIC" | criterion =="BIC" | criterion =="r-adj" | criterion =="PRESS"){
+                    if(direction=="backward")
+                    {
+                    	fit = fit2
+                    	if (result == TRUE) ## if the p-values of the new model are below cutoff, stop and return new model
+                    		switch = FALSE
+                    	else
+                    		switch = TRUE
+                        	
+                    }else if(direction == "both"){
+                        fit <- fit2
+                        switch <- result #send correction1 result as switch
                     }
                 }
                 
@@ -466,6 +496,10 @@ function(fit,scope=eval(fit$call$data), alpha = 0.05,direction = "forward",crite
             crit_forward <- -summary(fit_forward)$"adj.r.squared" #save -adjusted r-square
             crit_backward <- -summary(fit_backward)$"adj.r.squared" #save -adjusted r-square
             crit_current <- -summary(fit_current)$"adj.r.squared" #save -adjusted r-square
+        } else if (criterion == "PRESS"){
+            crit_forward <- PRESS(fit_forward) #save -PRESS
+            crit_backward <- PRESS(fit_backward) #save -adjusted r-square
+            crit_current <- PRESS(fit_current) #save -adjusted r-square
         } else if (criterion == "p-value"){
             crit_forward <- max(drop1(fit_forward, test="F")$"Pr(>F)"[-1]) #save the max p-values of each model
             
@@ -540,7 +574,7 @@ function(fit,scope=eval(fit$call$data), alpha = 0.05,direction = "forward",crite
             fit <- list1[[1]]
             switch <- list1[[2]]
             out_tab <- list1[[3]]
-            if(trace==TRUE & switch == TRUE){cat("\n"); print(out_tab); print(fit); cat("\n\n")}
+            if(trace==TRUE){cat("\n"); print(out_tab); print(fit); cat("\n\n")}
         }
     }
     #############stepwise process####################
@@ -560,7 +594,41 @@ function(fit,scope=eval(fit$call$data), alpha = 0.05,direction = "forward",crite
         switch <- list1[[2]]
         forward_out_tab <- list1[[3]]
         backward_out_tab <- list1[[4]]
-        if(trace==TRUE){cat("\n"); if(is.table(forward_out_tab)==TRUE){print(forward_out_tab)}; print(backward_out_tab); print(fit); cat("\n\n")}
+        
+        print_stepwise_tables <- function(forward_out_tab, backward_out_tab, crit_column_to_sort)
+        {
+   	    	index_none = which(rownames(backward_out_tab) == "<none>")
+    	    var = c(rownames(backward_out_tab)[-index_none], rownames(forward_out_tab))
+            out_tab <- matrix(nrow=length(var), ncol=10) #outcome table
+            colnames(out_tab) <- c("RSS", "AIC", "BIC", "adj.rsq","PRESS", "max_pvalue", "max VIF", "alpha_cut-off", "Bonferroni", "FDR")
+            rownames(out_tab) <- paste(var)    	    	
+    	    	
+    	    out_tab[,1:7] = matrix(as.numeric(rbind(backward_out_tab[-which(rownames(backward_out_tab) == "<none>"),1:7],forward_out_tab[,1:7])), nrow = length(c(rownames(backward_out_tab), rownames(forward_out_tab)))-1)
+            out_tab = as.table(out_tab[order(as.numeric(out_tab[,crit_column_to_sort])),])
+    	    	
+            out_tab[,8:10] <- as.character(as.logical(c(rbind(forward_out_tab[,8:10],backward_out_tab[-which(rownames(backward_out_tab) == "<none>"),8:10]) == "TRUE")))
+            
+            cat("\n")        	    	
+        	print(out_tab)    
+            cat("\n")        	    	
+        	print(fit)    	
+        	cat("\n\n")
+        }
+        
+        
+        if(trace==TRUE)
+        {
+        	cat("\n"); 
+	        if(is.table(forward_out_tab)==TRUE)
+    	    {
+        		print_stepwise_tables(forward_out_tab, backward_out_tab, crit_column_to_sort)
+	        }else
+	        {
+    	    	print(backward_out_tab)
+	        	print(fit); cat("\n\n")
+	        }
+    	    	
+        }
         
         while (switch==TRUE){
             list1 <- stepwise(fit)
@@ -568,7 +636,9 @@ function(fit,scope=eval(fit$call$data), alpha = 0.05,direction = "forward",crite
             switch <- list1[[2]]
             forward_out_tab <- list1[[3]]
             backward_out_tab <- list1[[4]]
-            if(trace==TRUE){cat("\n"); print(forward_out_tab); print(backward_out_tab); print(fit); cat("\n\n")}
+            if(trace==TRUE)
+            	print_stepwise_tables(forward_out_tab, backward_out_tab, crit_column_to_sort)
+
         }
         fit <- list1[[1]]
     }
