@@ -4,8 +4,8 @@ function(fit, scope=eval(fit$call$data), alpha = 0.05, criterion = "p-value", co
 drop1SignifReg.default <- function(fit, scope=eval(fit$call$data), alpha = 0.05, criterion = "p-value", correction = "FDR", override = FALSE){
     
     #####errors messages for wrong inputs#####
-    if(criterion != "p-value" && criterion != "AIC" && criterion != "BIC" && criterion != "r-adj"){
-        stop("\ncriterion should be one of the following: p-value, AIC, BIC, and r-adj\n")
+    if(criterion != "p-value" && criterion != "AIC" && criterion != "BIC" && criterion != "r-adj" && criterion != "PRESS"){
+        stop("\ncriterion should be one of the following: p-value, AIC, BIC, r-adj, or PRESS\n")
     }
     if(correction != "FDR" && correction != "Bonferroni" && correction !="Bonf" && correction != "None"){
         stop("\ncorrection should be one of the following: FDR, Bonferroni, Bonf, None\n")
@@ -41,6 +41,58 @@ drop1SignifReg.default <- function(fit, scope=eval(fit$call$data), alpha = 0.05,
         return(result) #when satisfy corrected criterion return result=TRUE, otherwise result=FALSE
     }
     
+ 
+    #######################PRESS residuals########################
+	PRESS <- function(linear.model) {
+	  #' calculate the predictive residuals
+	  pr <- residuals(linear.model)/(1-lm.influence(linear.model)$hat)
+	  #' calculate the PRESS
+	  PRESS <- sum(pr^2)
+	  return(PRESS)
+	}
+
+    ####################### VIF ########################
+	vif <- function(mod, ...) 
+	{
+	    if (any(is.na(coef(mod)))) 
+	        stop ("there are aliased coefficients in the model")
+	    v <- vcov(mod)
+	    assign <- attr(model.matrix(mod), "assign")
+	    if (names(coefficients(mod)[1]) == "(Intercept)") {
+	        v <- v[-1, -1]
+	        assign <- assign[-1]
+	    }
+	    else warning("No intercept: vifs may not be sensible.")
+	    terms <- labels(terms(mod))
+	    n.terms <- length(terms)
+	    if (n.terms < 2) stop("model contains fewer than 2 terms")
+	    R <- cov2cor(v)
+	    detR <- det(R)
+	    result <- matrix(0, n.terms, 3)
+	    rownames(result) <- terms
+	    colnames(result) <- c("GVIF", "Df", "GVIF^(1/(2*Df))")
+	    for (term in 1:n.terms) {
+	        subs <- which(assign == term)
+	        result[term, 1] <- det(as.matrix(R[subs, subs])) *
+	            det(as.matrix(R[-subs, -subs])) / detR
+	        result[term, 2] <- length(subs)
+	    }
+	    if (all(result[, 2] == 1)) result <- result[, 1]
+	    else result[, 3] <- result[, 1]^(1/(2 * result[, 2]))
+	    result
+	}
+
+	if(criterion == "p-value")
+		crit_column_to_sort = 6 ## this is used to sort and present the out_tab (ouput table) sorted by this criterion
+	else if(criterion == "AIC")
+		crit_column_to_sort = 2
+	else if(criterion == "BIC") 
+		crit_column_to_sort = 3
+	else if(criterion == "r-adj")
+		crit_column_to_sort = 4
+    else if(criterion == "PRESS")
+	    crit_column_to_sort = 5
+
     
     #########################drop1 proccess############################
     data <- eval(fit$call$data) #save data
@@ -63,19 +115,25 @@ drop1SignifReg.default <- function(fit, scope=eval(fit$call$data), alpha = 0.05,
         }else stop("\nwrong scope\n")
     }else stop("\nwrong scope\n")
     
-    out_tab <- matrix(nrow=length(var)+1, ncol=8) #outcome table
-    colnames(out_tab) <- c("RSS", "AIC", "BIC", "adj.rsq","max_pvalue", "alpha_cut-off", "Bonferroni", "FDR")
+    out_tab <- matrix(nrow=length(var)+1, ncol=10) #outcome table
+                colnames(out_tab) <- c("RSS", "AIC", "BIC", "adj.rsq","PRESS", "max_pvalue", "max VIF", "alpha_cut-off", "Bonferroni", "FDR")
     rownames(out_tab) <- c("<none>", paste("-", var))
     out_tab[1,1] <- deviance(fit) #RSS of the current model
     out_tab[1,2] <- AIC(fit) #AIC of the current model
     out_tab[1,3] <- BIC(fit) #BIC of the current model
     out_tab[1,4] <- summary(fit)$"adj.r.squared" #adjusted r-square of the current model
+    out_tab[1,5] <- PRESS(fit) #PRESS of the current model
+                
+    if (length(fit$coefficients) > 2) ## one of them is beta0, so need 3 or more coef
+        out_tab[1,7] <- max(vif(fit)) 
+    else
+        out_tab[1,7] <- NA
     
     fit_pval <- drop1(fit, test="F")$"Pr(>F)"[-1] #save p-values of the current model
-    out_tab[1,5] <- max(fit_pval) #the max p-value of the current model
-    out_tab[1,6] <- none(fit_pval) #pvalue cut-off (no correction)
-    out_tab[1,7] <- bonferroni(fit_pval) #Bonfferoni cut-off
-    out_tab[1,8] <- fdr(fit_pval) #FDR cut-off
+    out_tab[1,6] <- max(fit_pval) #the max p-value of the current model
+    out_tab[1,8] <- none(fit_pval) #pvalue cut-off (no correction)
+    out_tab[1,9] <- bonferroni(fit_pval) #Bonfferoni cut-off
+    out_tab[1,10] <- fdr(fit_pval) #FDR cut-off
     
     crit_value <- matrix() #criterion values
     if(criterion == "AIC"){
@@ -84,6 +142,8 @@ drop1SignifReg.default <- function(fit, scope=eval(fit$call$data), alpha = 0.05,
         crit_value[1] <- out_tab[1,3] #save BIC as criterion value #########
     } else if(criterion == "r-adj"){
         crit_value[1] <- -out_tab[1,4] #save -adjusted r-square
+    } else if(criterion == "PRESS"){
+        crit_value[1] <- out_tab[1,5] #save -adjusted r-square
     }
     
     #reg <- paste(response,"~",paste(reg_var,collapse="+"))
@@ -94,27 +154,36 @@ drop1SignifReg.default <- function(fit, scope=eval(fit$call$data), alpha = 0.05,
         out_tab[n+1,2] <- AIC(fit2) #save value of AIC value for each model ##########
         out_tab[n+1,3] <- BIC(fit2) #save value of BIC value for each model ##########
         out_tab[n+1,4] <- summary(fit2)$"adj.r.squared" #save adjusted r-square
+        out_tab[n+1,5] <- PRESS(fit2) #save adjusted r-square
+
+        if (length(fit2$coefficients) > 2) ## one of them is beta0, so need 3 or more coef
+           	out_tab[n+1,7] <- max(vif(fit2)) 
+        else
+           	out_tab[n+1,7] <- NA
         
         fit2_pval <- drop1(fit2, test="F")$"Pr(>F)"[-1] #save p-values of each model
         if(length(fit2_pval)==0)  stop("\nwrong scope\n")
-        out_tab[n+1,5] <- max(fit2_pval) #the max p-value of each model
-        out_tab[n+1,6] <- none(fit2_pval) #pvalue cut-off (no correction)
-        out_tab[n+1,7] <- bonferroni(fit2_pval) #Bonfferoni cut-off
-        out_tab[n+1,8] <- fdr(fit2_pval) #FDR cut-off
+        out_tab[n+1,6] <- max(fit2_pval) #the max p-value of each model
+        out_tab[n+1,8] <- none(fit2_pval) #pvalue cut-off (no correction)
+        out_tab[n+1,9] <- bonferroni(fit2_pval) #Bonfferoni cut-off
+        out_tab[n+1,10] <- fdr(fit2_pval) #FDR cut-off
         
         if (criterion == "p-value"){
-            crit_value[n+1] <- out_tab[n+1,5] #save the biggest p-value of each model
+            crit_value[n+1] <- out_tab[n+1,6] #save the biggest p-value of each model
         } else if (criterion == "AIC"){
             crit_value[n+1] <- out_tab[n+1,2] #save AIC value of each model ##########
         } else if (criterion == "BIC"){
             crit_value[n+1] <- out_tab[n+1,3] #save BIC value of each model ##########
         } else if (criterion == "r-adj"){
             crit_value[n+1] <- -out_tab[n+1,4] #save -adjusted r-square
+        } else if (criterion == "PRESS"){
+            crit_value[n+1] <- out_tab[n+1,5] #save -adjusted r-square
         }
     }
     
+	out_tab = out_tab[order(as.numeric(out_tab[,crit_column_to_sort])),]
     out_tab <- round(as.table(out_tab, dimnames),5)
-    out_tab[,6:8] <- as.character(as.logical(out_tab[,6:8]))
+    out_tab[,8:10] <- as.character(as.logical(out_tab[,8:10]))
     print(out_tab)
     
     crit_value[which(is.na(crit_value=="NA"))]  <- 99 #remove the model of which p-value is NA
@@ -139,15 +208,23 @@ drop1SignifReg.default <- function(fit, scope=eval(fit$call$data), alpha = 0.05,
             result_current_model <- none(pvalues_current_model)
         }
         if(criterion == "p-value"){
-            if(result_current_model == TRUE){
-                fit <- fit
-            }else{
+            if(result_current_model == FALSE){
                 fit <- fit2
-            }
-        }else if(criterion == "AIC" | criterion =="BIC" | criterion =="r-adj"){
-            if((result_current_model == FALSE) && (min(crit_value[-1]) > crit_value[1])){
-                fit <- fit2
-            }else fit <- fit
+            }else
+           	{
+	            if ((min(crit_value[-1]) < crit_value[1]) && (result == TRUE)) ## removing the predictor reduces the criterion
+           			fit = fit2
+           	}
+        }else if(criterion == "AIC" | criterion =="BIC" | criterion =="r-adj" | criterion =="PRESS")
+        {
+           	if (result_current_model == FALSE)
+           		fit = fit2
+           	else ## current model is true -> remove predictor only if reduces criterion AND new model is also TRUE
+           	{
+	            if ((min(crit_value[-1]) < crit_value[1]) && (result == TRUE)) ## removing the predictor reduces the criterion
+           			fit = fit2
+           	}
+
         }
     }else if(override == TRUE){ #do not check correction)
         fit <- fit2
